<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;

class CompositeQuoteContext implements QuoteContextInterface
{
    private readonly iterable $prioritizedContexts;

    public function __construct(iterable $prioritizedContexts)
    {
        $this->prioritizedContexts = $prioritizedContexts;
    }

    public function getQuote(): QuoteInterface
    {
        /** @var QuoteContextInterface $quoteContext */
        foreach ($this->prioritizedContexts as $quoteContext) {
            try {
                return $quoteContext->getQuote();
            } catch (QuoteNotFoundException $exception) {
                continue;
            }
        }

        throw new QuoteNotFoundException();
    }
}
