<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Context;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Exception\QuoteNotFoundException;
use Dedi\SyliusQuotePlugin\Storage\QuoteStorageInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Channel\Context\ChannelNotFoundException;

class SessionAndChannelBasedQuoteContext implements QuoteContextInterface
{
    private readonly ChannelContextInterface $channelContext;

    private readonly QuoteStorageInterface $quoteStorage;

    public function __construct(
        ChannelContextInterface $channelContext,
        QuoteStorageInterface $quoteStorage,
    ) {
        $this->channelContext = $channelContext;
        $this->quoteStorage = $quoteStorage;
    }

    public function getQuote(): QuoteInterface
    {
        try {
            $channel = $this->channelContext->getChannel();
        } catch (ChannelNotFoundException $exception) {
            throw new QuoteNotFoundException(null, $exception);
        }

        if (!$this->quoteStorage->hasForChannel($channel)) {
            throw new QuoteNotFoundException('Sylius was not able to find the quote in session');
        }

        $quote = $this->quoteStorage->getForChannel($channel);
        if (null === $quote) {
            $this->quoteStorage->removeForChannel($channel);

            throw new QuoteNotFoundException('Sylius was not able to find the quote in session');
        }

        return $quote;
    }
}
