<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\DependencyInjection;

use Dedi\SyliusQuotePlugin\Controller\QuoteController;
use Dedi\SyliusQuotePlugin\Controller\QuoteItemController;
use Dedi\SyliusQuotePlugin\Entity\Quote;
use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteInternalComment;
use Dedi\SyliusQuotePlugin\Entity\QuoteInternalCommentInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItem;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Form\Type\QuoteInternalCommentType;
use Dedi\SyliusQuotePlugin\Form\Type\QuoteItemType;
use Dedi\SyliusQuotePlugin\Form\Type\QuoteType;
use Dedi\SyliusQuotePlugin\Repository\QuoteRepository;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Sylius\Bundle\ResourceBundle\SyliusResourceBundle;
use Sylius\Component\Resource\Factory\Factory;
use Symfony\Component\Config\Definition\Builder\ArrayNodeDefinition;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_quote');
        $rootNode = $treeBuilder->getRootNode();

        $rootNode
            ->addDefaultsIfNotSet()
            ->children()
                ->scalarNode('driver')->defaultValue(SyliusResourceBundle::DRIVER_DOCTRINE_ORM)->end()
                ->booleanNode('allow_anonymous_quote')->defaultValue(false)->end()
                ->booleanNode('allow_cart_edit')->defaultValue(false)->end()
            ->end()
        ;

        $this->addResourcesSection($rootNode);

        return $treeBuilder;
    }

    private function addResourcesSection(ArrayNodeDefinition $nodeDefinition): void
    {
        $nodeDefinition
            ->children()
                ->arrayNode('resources')
                    ->addDefaultsIfNotSet()
                        ->children()
                        ->arrayNode('quote')
                            ->addDefaultsIfNotSet()
                            ->children()
                                ->variableNode('options')->end()
                                ->arrayNode('classes')
                                    ->addDefaultsIfNotSet()
                                    ->children()
                                        ->scalarNode('model')->defaultValue(Quote::class)->cannotBeEmpty()->end()
                                        ->scalarNode('interface')->defaultValue(QuoteInterface::class)->cannotBeEmpty()->end()
                                        ->scalarNode('factory')->defaultValue(Factory::class)->cannotBeEmpty()->end()
                                        ->scalarNode('repository')->defaultValue(QuoteRepository::class)->cannotBeEmpty()->end()
                                        ->scalarNode('controller')->defaultValue(QuoteController::class)->cannotBeEmpty()->end()
                                        ->scalarNode('form')->defaultValue(QuoteType::class)->cannotBeEmpty()->end()
                                    ->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('quote_item')
                            ->addDefaultsIfNotSet()
                            ->children()
                                ->variableNode('options')->end()
                                ->arrayNode('classes')
                                    ->addDefaultsIfNotSet()
                                    ->children()
                                        ->scalarNode('model')->defaultValue(QuoteItem::class)->cannotBeEmpty()->end()
                                        ->scalarNode('interface')->defaultValue(QuoteItemInterface::class)->cannotBeEmpty()->end()
                                        ->scalarNode('factory')->defaultValue(Factory::class)->cannotBeEmpty()->end()
                                        ->scalarNode('controller')->defaultValue(QuoteItemController::class)->cannotBeEmpty()->end()
                                        ->scalarNode('form')->defaultValue(QuoteItemType::class)->cannotBeEmpty()->end()
                                    ->end()
                                ->end()
                            ->end()
                        ->end()
                        ->arrayNode('quote_internal_comment')
                            ->addDefaultsIfNotSet()
                            ->children()
                                ->variableNode('options')->end()
                                ->arrayNode('classes')
                                    ->addDefaultsIfNotSet()
                                    ->children()
                                        ->scalarNode('model')->defaultValue(QuoteInternalComment::class)->cannotBeEmpty()->end()
                                        ->scalarNode('interface')->defaultValue(QuoteInternalCommentInterface::class)->cannotBeEmpty()->end()
                                        ->scalarNode('factory')->defaultValue(Factory::class)->cannotBeEmpty()->end()
                                        ->scalarNode('controller')->defaultValue(ResourceController::class)->cannotBeEmpty()->end()
                                        ->scalarNode('form')->defaultValue(QuoteInternalCommentType::class)->cannotBeEmpty()->end()
                                    ->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ->end()
        ;
    }
}
