<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Service;

use Dedi\SyliusQuotePlugin\Enum\QuoteTransitionEnum;
use Dedi\SyliusQuotePlugin\Model\OrderInterface;
use Doctrine\ORM\EntityManagerInterface;
use SM\Factory\Factory;

class QuoteValidatedStateApplicator
{
    private readonly Factory $stateMachineFactory;

    private readonly EntityManagerInterface $entityManager;

    public function __construct(
        Factory $stateMachineFactory,
        EntityManagerInterface $entityManager,
    ) {
        $this->stateMachineFactory = $stateMachineFactory;
        $this->entityManager = $entityManager;
    }

    public function apply(OrderInterface $order): void
    {
        $quote = $order->getQuote();
        if (null === $quote) {
            return;
        }

        $stateMachine = $this->stateMachineFactory->get($quote, QuoteTransitionEnum::GRAPH);

        $stateMachine->apply(QuoteTransitionEnum::TRANSITIONS_VALIDATE);

        $this->entityManager->persist($quote);
    }
}
